package com.fhkj.oltinspection.util;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.TimeoutException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.fhkj.oltinspection.exception.OltException;
import com.fhkj.oltinspection.pojo.Result;
import com.fhkj.oltinspection.pojo.SshConnection;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelShell;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;

/**
 * @author 胡懿
 * @see 这个类提供shhz相关操作，这个类不提供空的构造方法，只有传入基本的连接参数才能初始化完成
 */
public class SSHUtil {

    private SshConnection sshConnection;

    private Channel channel = null;

    private InputStream ins = null;

    private DataOutputStream dos = null;

    private Map<String, Object> sshConfigMap = (Map<String, Object>) ReadProperties.sshConfigMap;

    public SSHUtil(SshConnection sshConnection) {
        super();
        Result result = validateSshConnectionParameter(sshConnection);
        if (!(boolean) result.getState()) {
            throw new OltException(result.getMsg());
        }
        this.sshConnection = sshConnection;
    }

    /**
     * @return the sshConnection
     */
    public SshConnection getSshConnection() {
        return sshConnection;
    }

    /**
     * @param sshConnection the sshConnection to set
     */
    public void setSshConnection(SshConnection sshConnection) {
        this.sshConnection = sshConnection;
    }

    /**
     * @param sshConnection
     * @return Result
     * @see 该方法用于判断连接参数是否合法
     */
    private Result validateSshConnectionParameter(SshConnection sshConnection) {
        if (null == sshConnection.getHost() || "".equals(sshConnection.getHost())) {
            return new Result("IP地址为空", false);
        } else if (!validateHost(sshConnection.getHost())) {
            return new Result("ip地址不合法", false);
        }
        if (null == sshConnection.getPort() || "0".equals(sshConnection.getPort().toString())) {
            return new Result("端口号为空", false);
        }
        if (null == sshConnection.getUsername() || "".equals(sshConnection.getUsername())) {
            return new Result("用户名为空", false);
        }
        if (null == sshConnection.getPassword() || "".equals(sshConnection.getPassword())) {
            return new Result("密码为空", false);
        }
        return new Result("验证成功", true);
    }

    /**
     * @param ip
     * @return boolean
     * @see 正则验证ip地址是否合法
     */
    private boolean validateHost(String ip) {
        String pattern = "(2(5[0-5]{1}|[0-4]\\d{1})|[0-1]?\\d{1,2})(\\.(2(5[0-5]{1}|[0-4]\\d{1})|[0-1]?\\d{1,2})){3}";
        Pattern r = Pattern.compile(pattern);
        Matcher m = r.matcher(ip);
        return m.matches();
    }

    /**
     * @return Result
     * @see 打开连接的方法，该方法调用时，需要先通过构造方法设置连接参数SshConnection
     */
    public Result initConnection() {
        String host = this.sshConnection.getHost();
        Integer port = this.sshConnection.getPort();
        String username = this.sshConnection.getUsername();
        String password = this.sshConnection.getPassword();
        Session session = null; // 打开连接
        JSch jsch = new JSch();
        Properties config = new Properties();
        try {
            session = jsch.getSession(username, host, port);
            session.setPassword(password);
            config.put("StrictHostKeyChecking", "no");
            session.setConfig(config);
            session.setTimeout((int) sshConfigMap.get("connectionTimeout"));
            session.connect();
            channel = session.openChannel("shell");
            ((ChannelShell) channel).setPtyType("vt100", 80, 100, 640, 960);
            channel.connect();
        } catch (JSchException e) {
            return new Result("连接异常", PackageExceptionUtil.exceptionToString(e), false);
        }
        return new Result("连接成功", true);
    }

    /**
     * @see 初始化对设备的输入输出流
     */
    public Result initInputAndOutput() {
        if (null == channel) {
            throw new OltException("channel为空，请先初始化连接");
        }
        try {
            this.ins = channel.getInputStream();
            this.dos = new DataOutputStream(channel.getOutputStream());
            this.dos.writeBytes("cd service" + "\r\n");
            this.dos.writeBytes("terminal length 0" + "\r\n");
            this.dos.writeBytes("cd .." + "\r\n");
        } catch (IOException e) {
            return new Result("开启成功", PackageExceptionUtil.exceptionToString(e), false);
        }
        return new Result("开启成功", true);
    }

    /**
     * @param command(命令),isRead(是否读取设备对该命令的返回),timeout(该命令最长等待放回时间)
     * @return Result
     * @see 用这个方法向设备输入命令，参数command是输入的命令，不需要加\r\n,已经加上了,
     * isRead代表是否读取设备的返回，true的时候会返回，timeout代表最长等待时间
     */
    public Result sendCommand(String command, boolean isRead, long timeout) {
        if (null == this.ins || null == this.dos) {
            throw new OltException("请先调用initInputAndOutput()方法初始化输入输出流");
        }
        try {
            this.dos.writeBytes(command + "\r\n"); // 输入命令
            this.dos.flush();
            Thread.sleep(2000);
        } catch (IOException | InterruptedException e) {
            return new Result("命令输入异常", PackageExceptionUtil.exceptionToString(e), false);
        }

        if (isRead) { // isRead是ture时读取设备返回的内容
            StringBuffer sb = new StringBuffer();
            try {
                byte[] buff = new byte[1024];
                int resRead = 0;
                long begin = System.currentTimeMillis();
                while (true) {
                    if (System.currentTimeMillis() > begin + timeout) {
                        throw new TimeoutException("read time out");
                    }
                    int len = this.ins.available();
                    if (len > 0) {
                        resRead = this.ins.read(buff);
                        if (resRead > 0) {
                            String str = new String(buff, 0, resRead);
                            sb.append(str);
                            if (sb.toString().trim().endsWith("#")) {
                                this.dos.writeBytes("clear" + "\r\n"); // 输入命令
                                this.dos.flush();
                                break;
                            }
                        }
                    } else {
                        Thread.sleep(1000);
                    }
                }
            } catch (IOException | InterruptedException | TimeoutException e) {
                return new Result("读取失败", PackageExceptionUtil.exceptionToString(e), false);
            }
            return new Result("读取成功", sb.toString(), true);
        } else {
            return new Result("输入成功", true);
        }
    }

    /**
     * @see 退出该设备
     */
    public Result sshExit() {
        try {
            this.dos.writeBytes("exit\r\n");
            this.dos.writeBytes("exit\r\n");
            this.dos.flush();
            this.dos.close();
            this.ins.close();
            this.channel = null;
            this.sshConnection = null;
            System.gc();
            ClearUtil.openExe();
        } catch (IOException e) {
            return new Result("退出失败", PackageExceptionUtil.exceptionToString(e), false);
        }
        return new Result("退出成功", true);
    }
}